import DiscRecording
from PyObjCTools.TestSupport import TestCase, expectedFailure, fourcc

DRTrackCallbackProc = b"i^{__DRTrack=}I^v"


class TestDRCoreTrack(TestCase):
    @expectedFailure
    def testCFTypes(self):
        self.assertIsCFType(DiscRecording.DRTrackRef)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.kDRTrackLengthKey, str)
        self.assertIsInstance(DiscRecording.kDRBlockSizeKey, str)
        self.assertIsInstance(DiscRecording.kDRBlockTypeKey, str)
        self.assertIsInstance(DiscRecording.kDRDataFormKey, str)
        self.assertIsInstance(DiscRecording.kDRSessionFormatKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackModeKey, str)
        self.assertIsInstance(DiscRecording.kDRVerificationTypeKey, str)
        self.assertIsInstance(DiscRecording.kDRDVDCopyrightInfoKey, str)
        self.assertIsInstance(DiscRecording.kDRDVDTimestampKey, str)
        self.assertIsInstance(DiscRecording.kDRBufferZone1DataKey, str)
        self.assertIsInstance(DiscRecording.kDRMaxBurnSpeedKey, str)
        self.assertIsInstance(DiscRecording.kDRPreGapLengthKey, str)
        self.assertIsInstance(DiscRecording.kDRPreGapIsRequiredKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackISRCKey, str)
        self.assertIsInstance(DiscRecording.kDRIndexPointsKey, str)
        self.assertIsInstance(DiscRecording.kDRAudioPreEmphasisKey, str)
        self.assertIsInstance(DiscRecording.kDRAudioFourChannelKey, str)
        self.assertIsInstance(DiscRecording.kDRSerialCopyManagementStateKey, str)
        self.assertIsInstance(DiscRecording.kDRVerificationTypeNone, str)
        self.assertIsInstance(DiscRecording.kDRVerificationTypeProduceAgain, str)
        self.assertIsInstance(DiscRecording.kDRVerificationTypeReceiveData, str)
        self.assertIsInstance(DiscRecording.kDRVerificationTypeChecksum, str)
        self.assertIsInstance(DiscRecording.kDRSCMSCopyrightFree, str)
        self.assertIsInstance(DiscRecording.kDRSCMSCopyrightProtectedOriginal, str)
        self.assertIsInstance(DiscRecording.kDRSCMSCopyrightProtectedCopy, str)
        self.assertIsInstance(DiscRecording.kDRNextWritableAddressKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackStartAddressKey, str)
        self.assertIsInstance(DiscRecording.kDRFreeBlocksKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackNumberKey, str)
        self.assertIsInstance(DiscRecording.kDRSessionNumberKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackTypeKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackIsEmptyKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackPacketTypeKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackPacketSizeKey, str)
        self.assertIsInstance(DiscRecording.kDRTrackTypeInvisible, str)
        self.assertIsInstance(DiscRecording.kDRTrackTypeIncomplete, str)
        self.assertIsInstance(DiscRecording.kDRTrackTypeReserved, str)
        self.assertIsInstance(DiscRecording.kDRTrackTypeClosed, str)
        self.assertIsInstance(DiscRecording.kDRTrackPacketTypeFixed, str)
        self.assertIsInstance(DiscRecording.kDRTrackPacketTypeVariable, str)
        self.assertIsInstance(DiscRecording.kDRBurnKey, str)
        self.assertIsInstance(DiscRecording.kDRSubchannelDataFormKey, str)
        self.assertIsInstance(DiscRecording.kDRSubchannelDataFormNone, str)
        self.assertIsInstance(DiscRecording.kDRSubchannelDataFormPack, str)
        self.assertIsInstance(DiscRecording.kDRSubchannelDataFormRaw, str)

        self.assertEqual(DiscRecording.kDRBlockSizeAudio, 2352)
        self.assertEqual(DiscRecording.kDRBlockSizeMode1Data, 2048)
        self.assertEqual(DiscRecording.kDRBlockSizeMode2Data, 2332)
        self.assertEqual(DiscRecording.kDRBlockSizeMode2Form1Data, 2048)
        self.assertEqual(DiscRecording.kDRBlockSizeMode2Form2Data, 2324)
        self.assertEqual(DiscRecording.kDRBlockSizeDVDData, 2048)

        self.assertEqual(DiscRecording.kDRBlockTypeAudio, 0)
        self.assertEqual(DiscRecording.kDRBlockTypeMode1Data, 8)
        self.assertEqual(DiscRecording.kDRBlockTypeMode2Data, 13)
        self.assertEqual(DiscRecording.kDRBlockTypeMode2Form1Data, 10)
        self.assertEqual(DiscRecording.kDRBlockTypeMode2Form2Data, 12)
        self.assertEqual(DiscRecording.kDRBlockTypeDVDData, 8)

        self.assertEqual(DiscRecording.kDRDataFormAudio, 0)
        self.assertEqual(DiscRecording.kDRDataFormMode1Data, 16)
        self.assertEqual(DiscRecording.kDRDataFormMode2Data, 32)
        self.assertEqual(DiscRecording.kDRDataFormMode2Form1Data, 32)
        self.assertEqual(DiscRecording.kDRDataFormMode2Form2Data, 32)
        self.assertEqual(DiscRecording.kDRDataFormDVDData, 16)

        self.assertEqual(DiscRecording.kDRTrackModeAudio, 0)
        self.assertEqual(DiscRecording.kDRTrackMode1Data, 4)
        self.assertEqual(DiscRecording.kDRTrackMode2Data, 4)
        self.assertEqual(DiscRecording.kDRTrackMode2Form1Data, 4)
        self.assertEqual(DiscRecording.kDRTrackMode2Form2Data, 4)
        self.assertEqual(DiscRecording.kDRTrackModeDVDData, 5)

        self.assertEqual(DiscRecording.kDRSessionFormatAudio, 0)
        self.assertEqual(DiscRecording.kDRSessionFormatMode1Data, 0)
        self.assertEqual(DiscRecording.kDRSessionFormatCDI, 0x10)
        self.assertEqual(DiscRecording.kDRSessionFormatCDXA, 0x20)
        self.assertEqual(DiscRecording.kDRSessionFormatDVDData, 0)

        self.assertEqual(DiscRecording.kDRFlagSubchannelDataRequested, 1 << 1)
        self.assertEqual(DiscRecording.kDRFlagNoMoreData, 1 << 0)

        self.assertEqual(DiscRecording.kDRTrackMessagePreBurn, fourcc(b"pre "))
        self.assertEqual(DiscRecording.kDRTrackMessageProduceData, fourcc(b"prod"))
        self.assertEqual(
            DiscRecording.kDRTrackMessageVerificationStarting, fourcc(b"vstr")
        )
        self.assertEqual(DiscRecording.kDRTrackMessageVerifyData, fourcc(b"vrfy"))
        self.assertEqual(DiscRecording.kDRTrackMessageVerificationDone, fourcc(b"vdon"))
        self.assertEqual(DiscRecording.kDRTrackMessagePostBurn, fourcc(b"post"))
        self.assertEqual(DiscRecording.kDRTrackMessageEstimateLength, fourcc(b"esti"))
        self.assertEqual(DiscRecording.kDRTrackMessageProducePreGap, fourcc(b"prpr"))
        self.assertEqual(DiscRecording.kDRTrackMessageVerifyPreGap, fourcc(b"vrpr"))

    def testStructs(self):
        # XXX: Needs manual work
        v = DiscRecording.DRTrackProductionInfo()
        self.assertEqual(v.buffer, None)
        self.assertEqual(v.reqCount, 0)
        self.assertEqual(v.actCount, 0)
        self.assertEqual(v.flags, 0)
        self.assertEqual(v.blockSize, 0)
        self.assertEqual(v.requestedAddress, 0)

    def testFunctions(self):
        self.assertIsInstance(DiscRecording.DRTrackGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DRTrackCreate)
        self.assertArgIsFunction(
            DiscRecording.DRTrackCreate, 1, DRTrackCallbackProc, True
        )

        DiscRecording.DRTrackSetProperties
        DiscRecording.DRTrackGetProperties
        DiscRecording.DRTrackSpeedTest
        DiscRecording.DRTrackEstimateLength



import DiscRecording
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestDRCoreDevice(TestCase):
    @expectedFailure
    def testCFTypes(self):
        self.assertIsCFType(DiscRecording.DRDeviceRef)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.kDRDeviceAppearedNotification, str)
        self.assertIsInstance(DiscRecording.kDRDeviceDisappearedNotification, str)
        self.assertIsInstance(DiscRecording.kDRDeviceStatusChangedNotification, str)
        self.assertIsInstance(DiscRecording.kDRDeviceSupportLevelKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceIORegistryEntryPathKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceVendorNameKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceProductNameKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceFirmwareRevisionKey, str)
        self.assertIsInstance(DiscRecording.kDRDevicePhysicalInterconnectKey, str)
        self.assertIsInstance(
            DiscRecording.kDRDevicePhysicalInterconnectLocationKey, str
        )
        self.assertIsInstance(DiscRecording.kDRDeviceWriteCapabilitiesKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceLoadingMechanismCanEjectKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceLoadingMechanismCanInjectKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceLoadingMechanismCanOpenKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceWriteBufferSizeKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceSupportLevelNone, str)
        self.assertIsInstance(DiscRecording.kDRDeviceSupportLevelUnsupported, str)
        self.assertIsInstance(DiscRecording.kDRDeviceSupportLevelVendorSupported, str)
        self.assertIsInstance(DiscRecording.kDRDeviceSupportLevelAppleSupported, str)
        self.assertIsInstance(DiscRecording.kDRDeviceSupportLevelAppleShipping, str)
        self.assertIsInstance(DiscRecording.kDRDevicePhysicalInterconnectATAPI, str)
        self.assertIsInstance(
            DiscRecording.kDRDevicePhysicalInterconnectFibreChannel, str
        )
        self.assertIsInstance(DiscRecording.kDRDevicePhysicalInterconnectFireWire, str)
        self.assertIsInstance(DiscRecording.kDRDevicePhysicalInterconnectUSB, str)
        self.assertIsInstance(DiscRecording.kDRDevicePhysicalInterconnectSCSI, str)
        self.assertIsInstance(
            DiscRecording.kDRDevicePhysicalInterconnectLocationInternal, str
        )
        self.assertIsInstance(
            DiscRecording.kDRDevicePhysicalInterconnectLocationExternal, str
        )
        self.assertIsInstance(
            DiscRecording.kDRDevicePhysicalInterconnectLocationUnknown, str
        )
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDRKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDRWKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDRKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDRDualLayerKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDRWKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDRWDualLayerKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDRAMKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDPlusRKey, str)
        self.assertIsInstance(
            DiscRecording.kDRDeviceCanWriteDVDPlusRDoubleLayerKey, str
        )
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDPlusRWKey, str)
        self.assertIsInstance(
            DiscRecording.kDRDeviceCanWriteDVDPlusRWDoubleLayerKey, str
        )
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteBDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteBDRKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteBDREKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteHDDVDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteHDDVDRKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteHDDVDRDualLayerKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteHDDVDRAMKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteHDDVDRWKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteHDDVDRWDualLayerKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDTextKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteIndexPointsKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteISRCKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDTAOKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDSAOKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteCDRawKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanWriteDVDDAOKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanTestWriteCDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanTestWriteDVDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanUnderrunProtectCDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCanUnderrunProtectDVDKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceIsBusyKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceIsTrayOpenKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMaximumWriteSpeedKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceCurrentWriteSpeedKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaStateKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaInfoKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceBurnSpeedsKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceTrackRefsKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceTrackInfoKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaStateMediaPresent, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaStateInTransition, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaStateNone, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaBSDNameKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaIsBlankKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaIsAppendableKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaIsOverwritableKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaIsErasableKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaIsReservedKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaBlocksOverwritableKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaBlocksFreeKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaBlocksUsedKey, str)
        self.assertIsInstance(
            DiscRecording.kDRDeviceMediaDoubleLayerL0DataZoneBlocksKey, str
        )
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTrackCountKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaSessionCountKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaClassKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeKey, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaClassCD, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaClassDVD, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaClassBD, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaClassHDDVD, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaClassUnknown, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeCDROM, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeCDR, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeCDRW, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDROM, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDRAM, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDR, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDRDualLayer, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDRW, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDRWDualLayer, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDPlusR, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDPlusRDoubleLayer, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDPlusRW, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeDVDPlusRWDoubleLayer, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeBDR, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeBDRE, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeBDROM, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeHDDVDROM, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeHDDVDR, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeHDDVDRDualLayer, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeHDDVDRAM, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeHDDVDRW, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeHDDVDRWDualLayer, str)
        self.assertIsInstance(DiscRecording.kDRDeviceMediaTypeUnknown, str)
        self.assertIsInstance(DiscRecording.kDRDeviceBurnSpeedCD1x, float)
        self.assertIsInstance(DiscRecording.kDRDeviceBurnSpeedDVD1x, float)
        self.assertIsInstance(DiscRecording.kDRDeviceBurnSpeedBD1x, float)
        self.assertIsInstance(DiscRecording.kDRDeviceBurnSpeedHDDVD1x, float)
        self.assertIsInstance(DiscRecording.kDRDeviceBurnSpeedMax, float)

    def testFunctions(self):
        self.assertIsInstance(DiscRecording.DRDeviceGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DRCopyDeviceArray)

        self.assertResultIsCFRetained(DiscRecording.DRDeviceCopyDeviceForBSDName)

        self.assertResultIsCFRetained(
            DiscRecording.DRDeviceCopyDeviceForIORegistryEntryPath
        )

        self.assertResultIsBOOL(DiscRecording.DRDeviceIsValid)

        DiscRecording.DRDeviceOpenTray
        DiscRecording.DRDeviceCloseTray
        DiscRecording.DRDeviceEjectMedia
        DiscRecording.DRDeviceAcquireMediaReservation
        DiscRecording.DRDeviceReleaseMediaReservation
        DiscRecording.DRDeviceAcquireExclusiveAccess
        DiscRecording.DRDeviceReleaseExclusiveAccess

        self.assertResultIsCFRetained(DiscRecording.DRDeviceCopyInfo)

        self.assertResultIsCFRetained(DiscRecording.DRDeviceCopyStatus)

        DiscRecording.DRDeviceKPSForXFactor
        DiscRecording.DRDeviceXFactorForKPS

    def testMacros(self):
        import math

        self.assertEqual(
            DiscRecording.DRDeviceKPSForCDXFactor(2.5),
            2.5 * DiscRecording.kDRDeviceBurnSpeedCD1x,
        )
        self.assertEqual(
            DiscRecording.DRDeviceKPSForDVDXFactor(2.5),
            2.5 * DiscRecording.kDRDeviceBurnSpeedDVD1x,
        )
        self.assertEqual(
            DiscRecording.DRDeviceCDXFactorForKPS(2500),
            math.floor(2500 / DiscRecording.kDRDeviceBurnSpeedCD1x + 0.5),
        )
        self.assertEqual(
            DiscRecording.DRDeviceDVDXFactorForKPS(2500),
            math.floor(2500 / DiscRecording.kDRDeviceBurnSpeedDVD1x + 0.5),
        )



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRCoreStatus(TestCase):
    def testConstants(self):
        self.assertIsInstance(DiscRecording.kDRStatusStateKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusPercentCompleteKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusCurrentSpeedKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusCurrentSessionKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusCurrentTrackKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusTotalSessionsKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusTotalTracksKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusEraseTypeKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateNone, str)
        self.assertIsInstance(DiscRecording.kDRStatusStatePreparing, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateVerifying, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateDone, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateFailed, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateSessionOpen, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateTrackOpen, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateTrackWrite, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateTrackClose, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateSessionClose, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateFinishing, str)
        self.assertIsInstance(DiscRecording.kDRStatusStateErasing, str)
        self.assertIsInstance(DiscRecording.kDRStatusProgressInfoKey, str)
        self.assertIsInstance(DiscRecording.kDRStatusProgressCurrentKPS, str)
        self.assertIsInstance(DiscRecording.kDRStatusProgressCurrentXFactor, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase
import objc


class TestDRFileHelper(DiscRecording.NSObject):
    def calculateSizeOfFile_fork_estimating_(self, a, b, c):
        return 1

    def prepareFileForBurn_(self, a):
        return 1

    def produceFile_fork_intoBuffer_length_atAddress_blockSize_(self, a, b, c, d, e, f):
        return 1

    def prepareFileForVerification_(self, a):
        return 1


class TestDRFile(TestCase):
    def testMethods(self):
        self.assertResultHasType(
            TestDRFileHelper.calculateSizeOfFile_fork_estimating_, objc._C_ULNG_LNG
        )
        self.assertArgHasType(
            TestDRFileHelper.calculateSizeOfFile_fork_estimating_, 1, objc._C_UINT
        )
        self.assertArgIsBOOL(TestDRFileHelper.calculateSizeOfFile_fork_estimating_, 2)

        self.assertResultIsBOOL(TestDRFileHelper.prepareFileForBurn_)

        self.assertResultHasType(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            1,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            2,
            b"o^v",
        )
        self.assertArgSizeInArg(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            2,
            3,
        )
        self.assertArgHasType(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            3,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            4,
            objc._C_ULNG_LNG,
        )
        self.assertArgHasType(
            TestDRFileHelper.produceFile_fork_intoBuffer_length_atAddress_blockSize_,
            5,
            objc._C_UINT,
        )

        self.assertResultIsBOOL(TestDRFileHelper.prepareFileForVerification_)

    def testProtocols(self):
        objc.protocolNamed("DRFileDataProduction")

    def testConstants(self):
        self.assertIsInstance(DiscRecording.DRLinkTypeHardLink, str)
        self.assertIsInstance(DiscRecording.DRLinkTypeSymbolicLink, str)
        self.assertIsInstance(DiscRecording.DRLinkTypeFinderAlias, str)

        self.assertEqual(DiscRecording.DRFileForkData, 0)
        self.assertEqual(DiscRecording.DRFileForkResource, 1)



import DiscRecording
from PyObjCTools.TestSupport import TestCase, expectedFailure

DRNotificationCallback = (
    b"v^{__DRNotificationCenter=}^v^{__CFString=}@^{__CFDictionary=}"
)


class TestDRCoreNotifications(TestCase):
    @expectedFailure
    def testCFTypes(self):
        self.assertIsCFType(DiscRecording.DRNotificationCenterRef)

    def testFunctions(self):
        self.assertIsInstance(DiscRecording.DRNotificationCenterGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DRNotificationCenterCreate)

        self.assertResultIsCFRetained(
            DiscRecording.DRNotificationCenterCreateRunLoopSource
        )

        self.assertArgIsFunction(
            DiscRecording.DRNotificationCenterAddObserver,
            2,
            DRNotificationCallback,
            True,
        )

        DiscRecording.DRNotificationCenterRemoveObserver



import CoreFoundation
import DiscRecording
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestDRCoreCDText(TestCase):
    @expectedFailure
    def testCFTypes(self):
        self.assertIsCFType(DiscRecording.DRCDTextBlockRef)

    def testConstants(self):
        self.assertEqual(
            DiscRecording.kDRCDTextEncodingISOLatin1Modified,
            CoreFoundation.kCFStringEncodingISOLatin1,
        )
        self.assertEqual(
            DiscRecording.kDRCDTextEncodingASCII, CoreFoundation.kCFStringEncodingASCII
        )

        self.assertEqual(DiscRecording.kDRCDTextGenreCodeUnknown, 0x0001)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeAdultContemporary, 0x0002)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeAlternativeRock, 0x0003)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeChildrens, 0x0004)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeClassical, 0x0005)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeContemporaryChristian, 0x0006)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeCountry, 0x0007)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeDance, 0x0008)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeEasyListening, 0x0009)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeErotic, 0x000A)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeFolk, 0x000B)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeGospel, 0x000C)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeHipHop, 0x000D)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeJazz, 0x000E)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeLatin, 0x000F)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeMusical, 0x0010)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeNewAge, 0x0011)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeOpera, 0x0012)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeOperetta, 0x0013)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodePop, 0x0014)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeRap, 0x0015)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeReggae, 0x0016)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeRock, 0x0017)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeRhythmAndBlues, 0x0018)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeSoundEffects, 0x0019)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeSoundtrack, 0x001A)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeSpokenWord, 0x001B)
        self.assertEqual(DiscRecording.kDRCDTextGenreCodeWorldMusic, 0x001C)

        self.assertIsInstance(DiscRecording.kDRCDTextLanguageKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextCharacterCodeKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextCFStringEncodingKey, str)
        self.assertIsInstance(
            DiscRecording.kDRCDTextCopyrightAssertedForSpecialMessagesKey, str
        )
        self.assertIsInstance(DiscRecording.kDRCDTextCopyrightAssertedForNamesKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextCopyrightAssertedForTitlesKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextTitleKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextPerformerKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextSongwriterKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextComposerKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextArrangerKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextSpecialMessageKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextDiscIdentKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextGenreKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextGenreCodeKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextClosedKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextMCNISRCKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextTOCKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextTOC2Key, str)
        self.assertIsInstance(DiscRecording.kDRCDTextSizeKey, str)

    def testFunctions(self):
        self.assertResultIsCFRetained(
            DiscRecording.DRCDTextBlockCreateArrayFromPackList
        )

        self.assertIsInstance(DiscRecording.DRCDTextBlockGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DRCDTextBlockCreate)

        DiscRecording.DRCDTextBlockGetProperties
        DiscRecording.DRCDTextBlockGetTrackDictionaries
        DiscRecording.DRCDTextBlockSetTrackDictionaries
        DiscRecording.DRCDTextBlockGetValue
        DiscRecording.DRCDTextBlockSetValue
        DiscRecording.DRCDTextBlockFlatten



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRContentProperties(TestCase):
    def testConstants(self):
        self.assertIsInstance(DiscRecording.kDRISOLevel, str)
        self.assertIsInstance(DiscRecording.kDRVolumeSet, str)
        self.assertIsInstance(DiscRecording.kDRPublisher, str)
        self.assertIsInstance(DiscRecording.kDRDataPreparer, str)
        self.assertIsInstance(DiscRecording.kDRSystemIdentifier, str)
        self.assertIsInstance(DiscRecording.kDRApplicationIdentifier, str)
        self.assertIsInstance(DiscRecording.kDRCopyrightFile, str)
        self.assertIsInstance(DiscRecording.kDRAbstractFile, str)
        self.assertIsInstance(DiscRecording.kDRBibliographicFile, str)
        self.assertIsInstance(DiscRecording.kDRBlockSize, str)
        self.assertIsInstance(DiscRecording.kDRDefaultDate, str)
        self.assertIsInstance(DiscRecording.kDRVolumeCreationDate, str)
        self.assertIsInstance(DiscRecording.kDRVolumeModificationDate, str)
        self.assertIsInstance(DiscRecording.kDRVolumeCheckedDate, str)
        self.assertIsInstance(DiscRecording.kDRVolumeExpirationDate, str)
        self.assertIsInstance(DiscRecording.kDRVolumeEffectiveDate, str)
        self.assertIsInstance(DiscRecording.kDRISOMacExtensions, str)
        self.assertIsInstance(DiscRecording.kDRISORockRidgeExtensions, str)
        self.assertIsInstance(DiscRecording.kDRSuppressMacSpecificFiles, str)
        self.assertIsInstance(DiscRecording.kDRAllFilesystems, str)
        self.assertIsInstance(DiscRecording.kDRISO9660, str)
        self.assertIsInstance(DiscRecording.kDRISO9660LevelOne, str)
        self.assertIsInstance(DiscRecording.kDRISO9660LevelTwo, str)
        self.assertIsInstance(DiscRecording.kDRJoliet, str)
        self.assertIsInstance(DiscRecording.kDRHFSPlus, str)
        self.assertIsInstance(DiscRecording.kDRUDF, str)
        self.assertIsInstance(DiscRecording.kDRISO9660VersionNumber, str)
        self.assertIsInstance(DiscRecording.kDRInvisible, str)
        self.assertIsInstance(DiscRecording.kDRCreationDate, str)
        self.assertIsInstance(DiscRecording.kDRContentModificationDate, str)
        self.assertIsInstance(DiscRecording.kDRAttributeModificationDate, str)
        self.assertIsInstance(DiscRecording.kDRAccessDate, str)
        self.assertIsInstance(DiscRecording.kDRBackupDate, str)
        self.assertIsInstance(DiscRecording.kDRRecordingDate, str)
        self.assertIsInstance(DiscRecording.kDREffectiveDate, str)
        self.assertIsInstance(DiscRecording.kDRExpirationDate, str)
        self.assertIsInstance(DiscRecording.kDRPosixFileMode, str)
        self.assertIsInstance(DiscRecording.kDRPosixUID, str)
        self.assertIsInstance(DiscRecording.kDRPosixGID, str)
        self.assertIsInstance(DiscRecording.kDRHFSPlusTextEncodingHint, str)
        self.assertIsInstance(DiscRecording.kDRHFSPlusCatalogNodeID, str)
        self.assertIsInstance(DiscRecording.kDRMacFileType, str)
        self.assertIsInstance(DiscRecording.kDRMacFileCreator, str)
        self.assertIsInstance(DiscRecording.kDRMacWindowBounds, str)
        self.assertIsInstance(DiscRecording.kDRMacIconLocation, str)
        self.assertIsInstance(DiscRecording.kDRMacScrollPosition, str)
        self.assertIsInstance(DiscRecording.kDRMacWindowView, str)
        self.assertIsInstance(DiscRecording.kDRMacFinderFlags, str)
        self.assertIsInstance(DiscRecording.kDRMacExtendedFinderFlags, str)
        self.assertIsInstance(DiscRecording.kDRMacFinderHideExtension, str)
        self.assertIsInstance(DiscRecording.kDRUDFWriteVersion, str)
        self.assertIsInstance(DiscRecording.kDRUDFVersion102, str)
        self.assertIsInstance(DiscRecording.kDRUDFVersion150, str)
        self.assertIsInstance(DiscRecording.kDRUDFPrimaryVolumeDescriptorNumber, str)
        self.assertIsInstance(DiscRecording.kDRUDFVolumeSequenceNumber, str)
        self.assertIsInstance(DiscRecording.kDRUDFMaxVolumeSequenceNumber, str)
        self.assertIsInstance(DiscRecording.kDRUDFInterchangeLevel, str)
        self.assertIsInstance(DiscRecording.kDRUDFMaxInterchangeLevel, str)
        self.assertIsInstance(DiscRecording.kDRUDFApplicationIdentifierSuffix, str)
        self.assertIsInstance(DiscRecording.kDRUDFVolumeSetIdentifier, str)
        self.assertIsInstance(DiscRecording.kDRUDFVolumeSetTimestamp, str)
        self.assertIsInstance(DiscRecording.kDRUDFVolumeSetImplementationUse, str)
        self.assertIsInstance(DiscRecording.kDRUDFRealTimeFile, str)
        self.assertIsInstance(DiscRecording.kDRUDFExtendedFilePermissions, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRDevice(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(DiscRecording.DRDevice.isValid)
        self.assertResultIsBOOL(DiscRecording.DRDevice.openTray)
        self.assertResultIsBOOL(DiscRecording.DRDevice.closeTray)
        self.assertResultIsBOOL(DiscRecording.DRDevice.ejectMedia)
        self.assertResultIsBOOL(DiscRecording.DRDevice.acquireExclusiveAccess)
        self.assertResultIsBOOL(DiscRecording.DRDevice.isEqualToDevice_)
        self.assertResultIsBOOL(DiscRecording.DRDevice.writesCD)
        self.assertResultIsBOOL(DiscRecording.DRDevice.writesDVD)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsPresent)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsTransitioning)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsBusy)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsBlank)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsAppendable)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsOverwritable)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsErasable)
        self.assertResultIsBOOL(DiscRecording.DRDevice.mediaIsReserved)
        self.assertResultIsBOOL(DiscRecording.DRDevice.trayIsOpen)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.DRDeviceBurnSpeedCD1x, float)
        self.assertIsInstance(DiscRecording.DRDeviceBurnSpeedDVD1x, float)
        self.assertIsInstance(DiscRecording.DRDeviceBurnSpeedBD1x, float)
        self.assertIsInstance(DiscRecording.DRDeviceBurnSpeedHDDVD1x, float)
        self.assertIsInstance(DiscRecording.DRDeviceBurnSpeedMax, float)

        self.assertIsInstance(DiscRecording.DRDeviceAppearedNotification, str)
        self.assertIsInstance(DiscRecording.DRDeviceDisappearedNotification, str)
        self.assertIsInstance(DiscRecording.DRDeviceStatusChangedNotification, str)
        self.assertIsInstance(DiscRecording.DRDeviceSupportLevelKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceIORegistryEntryPathKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceWriteCapabilitiesKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceVendorNameKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceProductNameKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceFirmwareRevisionKey, str)
        self.assertIsInstance(DiscRecording.DRDevicePhysicalInterconnectKey, str)
        self.assertIsInstance(
            DiscRecording.DRDevicePhysicalInterconnectLocationKey, str
        )
        self.assertIsInstance(DiscRecording.DRDeviceLoadingMechanismCanEjectKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceLoadingMechanismCanInjectKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceLoadingMechanismCanOpenKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceWriteBufferSizeKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceSupportLevelNone, str)
        self.assertIsInstance(DiscRecording.DRDeviceSupportLevelUnsupported, str)
        self.assertIsInstance(DiscRecording.DRDeviceSupportLevelVendorSupported, str)
        self.assertIsInstance(DiscRecording.DRDeviceSupportLevelAppleSupported, str)
        self.assertIsInstance(DiscRecording.DRDeviceSupportLevelAppleShipping, str)
        self.assertIsInstance(DiscRecording.DRDevicePhysicalInterconnectATAPI, str)
        self.assertIsInstance(
            DiscRecording.DRDevicePhysicalInterconnectFibreChannel, str
        )
        self.assertIsInstance(DiscRecording.DRDevicePhysicalInterconnectFireWire, str)
        self.assertIsInstance(DiscRecording.DRDevicePhysicalInterconnectSCSI, str)
        self.assertIsInstance(DiscRecording.DRDevicePhysicalInterconnectUSB, str)
        self.assertIsInstance(
            DiscRecording.DRDevicePhysicalInterconnectLocationInternal, str
        )
        self.assertIsInstance(
            DiscRecording.DRDevicePhysicalInterconnectLocationExternal, str
        )
        self.assertIsInstance(
            DiscRecording.DRDevicePhysicalInterconnectLocationUnknown, str
        )
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDRKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDRWKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDRKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDRDualLayerKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDRWKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDRWDualLayerKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDRAMKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDPlusRKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDPlusRDoubleLayerKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDPlusRWKey, str)
        self.assertIsInstance(
            DiscRecording.DRDeviceCanWriteDVDPlusRWDoubleLayerKey, str
        )
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteBDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteBDRKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteBDREKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteHDDVDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteHDDVDRKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteHDDVDRDualLayerKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteHDDVDRAMKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteHDDVDRWKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteHDDVDRWDualLayerKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDTextKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteIndexPointsKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteISRCKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDTAOKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDSAOKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteCDRawKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanWriteDVDDAOKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanTestWriteCDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanTestWriteDVDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanUnderrunProtectCDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCanUnderrunProtectDVDKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceIsBusyKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceIsTrayOpenKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMaximumWriteSpeedKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceCurrentWriteSpeedKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaStateKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaInfoKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceBurnSpeedsKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceTrackRefsKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceTrackInfoKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaStateMediaPresent, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaStateInTransition, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaStateNone, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaBSDNameKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaIsBlankKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaIsAppendableKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaIsOverwritableKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaIsErasableKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaIsReservedKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaOverwritableSpaceKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaFreeSpaceKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaUsedSpaceKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaBlocksOverwritableKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaBlocksFreeKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaBlocksUsedKey, str)
        self.assertIsInstance(
            DiscRecording.DRDeviceMediaDoubleLayerL0DataZoneBlocksKey, str
        )
        self.assertIsInstance(DiscRecording.DRDeviceMediaTrackCountKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaSessionCountKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaClassKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeKey, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaClassCD, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaClassDVD, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaClassBD, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaClassHDDVD, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaClassUnknown, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeCDROM, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeCDR, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeCDRW, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDROM, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDRAM, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDR, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDRDualLayer, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDRW, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDRWDualLayer, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDPlusR, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDPlusRDoubleLayer, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDPlusRW, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeDVDPlusRWDoubleLayer, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeBDR, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeBDRE, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeBDROM, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeHDDVDROM, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeHDDVDR, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeHDDVDRDualLayer, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeHDDVDRAM, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeHDDVDRW, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeHDDVDRWDualLayer, str)
        self.assertIsInstance(DiscRecording.DRDeviceMediaTypeUnknown, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRErase(TestCase):
    def testConstants(self):
        self.assertIsInstance(DiscRecording.DREraseTypeKey, str)
        self.assertIsInstance(DiscRecording.DREraseTypeQuick, str)
        self.assertIsInstance(DiscRecording.DREraseTypeComplete, str)
        self.assertIsInstance(DiscRecording.DREraseStatusChangedNotification, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestDRCoreErase(TestCase):
    @expectedFailure
    def testCFTypes(self):
        self.assertIsCFType(DiscRecording.DREraseRef)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.kDREraseStatusChangedNotification, str)
        self.assertIsInstance(DiscRecording.kDREraseTypeKey, str)
        self.assertIsInstance(DiscRecording.kDREraseTypeQuick, str)
        self.assertIsInstance(DiscRecording.kDREraseTypeComplete, str)

    def testFunctions(self):
        self.assertIsInstance(DiscRecording.DREraseGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DREraseCreate)

        DiscRecording.DREraseStart

        self.assertResultIsCFRetained(DiscRecording.DREraseCopyStatus)

        DiscRecording.DREraseGetDevice
        DiscRecording.DREraseSetProperties
        DiscRecording.DREraseGetProperties



import DiscRecording
from PyObjCTools.TestSupport import TestCase, expectedFailure


class TestDRCoreBurn(TestCase):
    @expectedFailure
    def testCFTypes(self):
        self.assertIsCFType(DiscRecording.DRBurnRef)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.kDRBurnStatusChangedNotification, str)
        self.assertIsInstance(DiscRecording.kDRBurnRequestedSpeedKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnAppendableKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnOverwriteDiscKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnVerifyDiscKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnCompletionActionKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnUnderrunProtectionKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnTestingKey, str)
        self.assertIsInstance(DiscRecording.kDRSynchronousBehaviorKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnFailureActionKey, str)
        self.assertIsInstance(DiscRecording.kDRMediaCatalogNumberKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnDoubleLayerL0DataZoneBlocksKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnStrategyKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnStrategyIsRequiredKey, str)
        self.assertIsInstance(DiscRecording.kDRCDTextKey, str)
        self.assertIsInstance(DiscRecording.kDRBurnCompletionActionEject, str)
        self.assertIsInstance(DiscRecording.kDRBurnCompletionActionMount, str)
        self.assertIsInstance(DiscRecording.kDRBurnFailureActionEject, str)
        self.assertIsInstance(DiscRecording.kDRBurnFailureActionNone, str)
        self.assertIsInstance(DiscRecording.kDRBurnStrategyCDTAO, str)
        self.assertIsInstance(DiscRecording.kDRBurnFailureActionNone, str)
        self.assertIsInstance(DiscRecording.kDRBurnStrategyCDTAO, str)
        self.assertIsInstance(DiscRecording.kDRBurnStrategyCDSAO, str)
        self.assertIsInstance(DiscRecording.kDRBurnStrategyBDDAO, str)

    def testFunctions(self):
        self.assertIsInstance(DiscRecording.DRBurnGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DRBurnCreate)

        DiscRecording.DRBurnWriteLayout
        DiscRecording.DRBurnAbort

        self.assertResultIsCFRetained(DiscRecording.DRBurnCopyStatus)

        DiscRecording.DRBurnGetDevice
        DiscRecording.DRBurnSetProperties
        DiscRecording.DRBurnGetProperties



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRBurn(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(DiscRecording.DRBurn.appendable)
        self.assertArgIsBOOL(DiscRecording.DRBurn.setAppendable_, 0)

        self.assertResultIsBOOL(DiscRecording.DRBurn.verifyDisc)
        self.assertArgIsBOOL(DiscRecording.DRBurn.setVerifyDisc_, 0)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.DRBurnRequestedSpeedKey, str)
        self.assertIsInstance(DiscRecording.DRBurnAppendableKey, str)
        self.assertIsInstance(DiscRecording.DRBurnOverwriteDiscKey, str)
        self.assertIsInstance(DiscRecording.DRBurnVerifyDiscKey, str)
        self.assertIsInstance(DiscRecording.DRBurnCompletionActionKey, str)
        self.assertIsInstance(DiscRecording.DRBurnUnderrunProtectionKey, str)
        self.assertIsInstance(DiscRecording.DRBurnTestingKey, str)
        self.assertIsInstance(DiscRecording.DRSynchronousBehaviorKey, str)
        self.assertIsInstance(DiscRecording.DRBurnFailureActionKey, str)
        self.assertIsInstance(DiscRecording.DRMediaCatalogNumberKey, str)
        self.assertIsInstance(DiscRecording.DRBurnDoubleLayerL0DataZoneBlocksKey, str)
        self.assertIsInstance(DiscRecording.DRBurnStrategyKey, str)
        self.assertIsInstance(DiscRecording.DRBurnStrategyIsRequiredKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextKey, str)
        self.assertIsInstance(DiscRecording.DRBurnCompletionActionEject, str)
        self.assertIsInstance(DiscRecording.DRBurnCompletionActionMount, str)
        self.assertIsInstance(DiscRecording.DRBurnFailureActionEject, str)
        self.assertIsInstance(DiscRecording.DRBurnFailureActionNone, str)
        self.assertIsInstance(DiscRecording.DRBurnStrategyCDTAO, str)
        self.assertIsInstance(DiscRecording.DRBurnStrategyCDSAO, str)
        self.assertIsInstance(DiscRecording.DRBurnStrategyDVDDAO, str)
        self.assertIsInstance(DiscRecording.DRBurnStrategyBDDAO, str)
        self.assertIsInstance(DiscRecording.DRBurnStatusChangedNotification, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase, fourcc

DRFileProc = b"i^v^{__DRFile=}I^v"


class TestDRContentFile(TestCase):
    def testFunctions(self):
        self.assertIsInstance(DiscRecording.DRFileGetTypeID(), int)

        self.assertResultIsCFRetained(DiscRecording.DRFileCreateReal)
        self.assertArgIsIn(DiscRecording.DRFileCreateReal, 0)

        self.assertResultIsCFRetained(DiscRecording.DRFileCreateRealWithURL)

        self.assertResultIsCFRetained(DiscRecording.DRFileCreateVirtualWithData)
        self.assertArgIsIn(DiscRecording.DRFileCreateVirtualWithData, 1)
        self.assertArgSizeInArg(DiscRecording.DRFileCreateVirtualWithData, 1, 2)

        self.assertResultIsCFRetained(DiscRecording.DRFileCreateVirtualWithCallback)
        self.assertArgIsFunction(
            DiscRecording.DRFileCreateVirtualWithCallback, 1, DRFileProc, True
        )

        DiscRecording.DRFileCreateVirtualLink

    def testConstants(self):
        self.assertEqual(DiscRecording.kDRLinkTypeHardLink, 1)
        self.assertEqual(DiscRecording.kDRLinkTypeSymbolicLink, 2)
        self.assertEqual(DiscRecording.kDRLinkTypeFinderAlias, 3)

        self.assertEqual(DiscRecording.kDRFileMessageForkSize, fourcc(b"fsiz"))
        self.assertEqual(DiscRecording.kDRFileMessagePreBurn, fourcc(b"pre "))
        self.assertEqual(DiscRecording.kDRFileMessageProduceData, fourcc(b"prod"))
        self.assertEqual(
            DiscRecording.kDRFileMessageVerificationStarting, fourcc(b"vrfy")
        )
        self.assertEqual(DiscRecording.kDRFileMessagePostBurn, fourcc(b"post"))
        self.assertEqual(DiscRecording.kDRFileMessageRelease, fourcc(b"bye "))

        self.assertEqual(DiscRecording.kDRFileForkData, 0)
        self.assertEqual(DiscRecording.kDRFileForkResource, 1)

        self.assertEqual(DiscRecording.kDRFileForkSizeActual, 0)
        self.assertEqual(DiscRecording.kDRFileForkSizeEstimate, 1)

    def testStructs(self):
        v = DiscRecording.DRFileForkSizeInfo()
        self.assertEqual(v.fork, 0)
        self.assertEqual(v.query, 0)
        self.assertEqual(v.size, 0)

        # XXX: Requires manual work
        v = DiscRecording.DRFileProductionInfo()
        self.assertEqual(v.requestedAddress, 0)
        self.assertEqual(v.buffer, None)
        self.assertEqual(v.reqCount, 0)
        self.assertEqual(v.actCount, 0)
        self.assertEqual(v.blockSize, 0)
        self.assertEqual(v.fork, 0)



import DiscRecording
import Foundation
from PyObjCTools.TestSupport import TestCase


class TestDRCDText(TestCase):
    def testConstants(self):
        self.assertEqual(
            DiscRecording.DRCDTextEncodingISOLatin1Modified,
            Foundation.NSISOLatin1StringEncoding,
        )
        self.assertEqual(
            DiscRecording.DRCDTextEncodingASCII, Foundation.NSASCIIStringEncoding
        )

        self.assertEqual(DiscRecording.DRCDTextGenreCodeUnknown, 0x0001)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeAdultContemporary, 0x0002)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeAlternativeRock, 0x0003)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeChildrens, 0x0004)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeClassical, 0x0005)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeContemporaryChristian, 0x0006)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeCountry, 0x0007)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeDance, 0x0008)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeEasyListening, 0x0009)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeErotic, 0x000A)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeFolk, 0x000B)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeGospel, 0x000C)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeHipHop, 0x000D)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeJazz, 0x000E)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeLatin, 0x000F)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeMusical, 0x0010)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeNewAge, 0x0011)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeOpera, 0x0012)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeOperetta, 0x0013)
        self.assertEqual(DiscRecording.DRCDTextGenreCodePop, 0x0014)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeRap, 0x0015)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeReggae, 0x0016)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeRock, 0x0017)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeRhythmAndBlues, 0x0018)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeSoundEffects, 0x0019)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeSoundtrack, 0x001A)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeSpokenWord, 0x001B)
        self.assertEqual(DiscRecording.DRCDTextGenreCodeWorldMusic, 0x001C)

        self.assertIsInstance(DiscRecording.DRCDTextLanguageKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextCharacterCodeKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextNSStringEncodingKey, str)
        self.assertIsInstance(
            DiscRecording.DRCDTextCopyrightAssertedForSpecialMessagesKey, str
        )
        self.assertIsInstance(DiscRecording.DRCDTextCopyrightAssertedForNamesKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextCopyrightAssertedForTitlesKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextTitleKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextPerformerKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextSongwriterKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextComposerKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextArrangerKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextSpecialMessageKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextDiscIdentKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextGenreKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextGenreCodeKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextClosedKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextMCNISRCKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextTOCKey, str)
        self.assertIsInstance(DiscRecording.DRCDTextTOC2Key, str)
        self.assertIsInstance(DiscRecording.DRCDTextSizeKey, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRFSObject(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(DiscRecording.DRFSObject.isVirtual)

        self.assertArgIsBOOL(
            DiscRecording.DRFSObject.propertyForKey_inFilesystem_mergeWithOtherFilesystems_,
            2,
        )
        self.assertArgIsBOOL(
            DiscRecording.DRFSObject.propertiesForFilesystem_mergeWithOtherFilesystems_,
            1,
        )

    def testConstants(self):
        self.assertEqual(DiscRecording.DRFilesystemInclusionMaskISO9660, 1 << 0)
        self.assertEqual(DiscRecording.DRFilesystemInclusionMaskJoliet, 1 << 1)
        self.assertEqual(DiscRecording.DRFilesystemInclusionMaskUDF, 1 << 2)
        self.assertEqual(DiscRecording.DRFilesystemInclusionMaskHFSPlus, 1 << 3)

        self.assertIsInstance(DiscRecording.DRAllFilesystems, str)
        self.assertIsInstance(DiscRecording.DRISO9660, str)
        self.assertIsInstance(DiscRecording.DRISO9660LevelOne, str)
        self.assertIsInstance(DiscRecording.DRISO9660LevelTwo, str)
        self.assertIsInstance(DiscRecording.DRJoliet, str)
        self.assertIsInstance(DiscRecording.DRHFSPlus, str)
        self.assertIsInstance(DiscRecording.DRUDF, str)
        self.assertIsInstance(DiscRecording.DRISO9660VersionNumber, str)
        self.assertIsInstance(DiscRecording.DRInvisible, str)
        self.assertIsInstance(DiscRecording.DRCreationDate, str)
        self.assertIsInstance(DiscRecording.DRContentModificationDate, str)
        self.assertIsInstance(DiscRecording.DRAttributeModificationDate, str)
        self.assertIsInstance(DiscRecording.DRAccessDate, str)
        self.assertIsInstance(DiscRecording.DRBackupDate, str)
        self.assertIsInstance(DiscRecording.DREffectiveDate, str)
        self.assertIsInstance(DiscRecording.DRExpirationDate, str)
        self.assertIsInstance(DiscRecording.DRRecordingDate, str)
        self.assertIsInstance(DiscRecording.DRPosixFileMode, str)
        self.assertIsInstance(DiscRecording.DRPosixUID, str)
        self.assertIsInstance(DiscRecording.DRPosixGID, str)
        self.assertIsInstance(DiscRecording.DRHFSPlusTextEncodingHint, str)
        self.assertIsInstance(DiscRecording.DRHFSPlusCatalogNodeID, str)
        self.assertIsInstance(DiscRecording.DRMacFileType, str)
        self.assertIsInstance(DiscRecording.DRMacFileCreator, str)
        self.assertIsInstance(DiscRecording.DRMacWindowBounds, str)
        self.assertIsInstance(DiscRecording.DRMacIconLocation, str)
        self.assertIsInstance(DiscRecording.DRMacScrollPosition, str)
        self.assertIsInstance(DiscRecording.DRMacWindowView, str)
        self.assertIsInstance(DiscRecording.DRMacFinderFlags, str)
        self.assertIsInstance(DiscRecording.DRMacExtendedFinderFlags, str)
        self.assertIsInstance(DiscRecording.DRMacFinderHideExtension, str)
        self.assertIsInstance(DiscRecording.DRUDFWriteVersion, str)
        self.assertIsInstance(DiscRecording.DRUDFVersion102, str)
        self.assertIsInstance(DiscRecording.DRUDFVersion150, str)
        self.assertIsInstance(DiscRecording.DRUDFPrimaryVolumeDescriptorNumber, str)
        self.assertIsInstance(DiscRecording.DRUDFVolumeSequenceNumber, str)
        self.assertIsInstance(DiscRecording.DRUDFMaxVolumeSequenceNumber, str)
        self.assertIsInstance(DiscRecording.DRUDFInterchangeLevel, str)
        self.assertIsInstance(DiscRecording.DRUDFMaxInterchangeLevel, str)
        self.assertIsInstance(DiscRecording.DRUDFApplicationIdentifierSuffix, str)
        self.assertIsInstance(DiscRecording.DRUDFVolumeSetIdentifier, str)
        self.assertIsInstance(DiscRecording.DRUDFVolumeSetTimestamp, str)
        self.assertIsInstance(DiscRecording.DRUDFVolumeSetImplementationUse, str)
        self.assertIsInstance(DiscRecording.DRUDFRealTimeFile, str)
        self.assertIsInstance(DiscRecording.DRUDFExtendedFilePermissions, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRStatus(TestCase):
    def testConstants(self):
        self.assertIsInstance(DiscRecording.DRStatusStateKey, str)
        self.assertIsInstance(DiscRecording.DRStatusPercentCompleteKey, str)
        self.assertIsInstance(DiscRecording.DRStatusCurrentSessionKey, str)
        self.assertIsInstance(DiscRecording.DRStatusCurrentTrackKey, str)
        self.assertIsInstance(DiscRecording.DRStatusTotalSessionsKey, str)
        self.assertIsInstance(DiscRecording.DRStatusTotalTracksKey, str)
        self.assertIsInstance(DiscRecording.DRStatusCurrentSpeedKey, str)
        self.assertIsInstance(DiscRecording.DRStatusEraseTypeKey, str)
        self.assertIsInstance(DiscRecording.DRStatusStateNone, str)
        self.assertIsInstance(DiscRecording.DRStatusStatePreparing, str)
        self.assertIsInstance(DiscRecording.DRStatusStateVerifying, str)
        self.assertIsInstance(DiscRecording.DRStatusStateDone, str)
        self.assertIsInstance(DiscRecording.DRStatusStateFailed, str)
        self.assertIsInstance(DiscRecording.DRStatusStateSessionOpen, str)
        self.assertIsInstance(DiscRecording.DRStatusStateTrackOpen, str)
        self.assertIsInstance(DiscRecording.DRStatusStateTrackWrite, str)
        self.assertIsInstance(DiscRecording.DRStatusStateTrackClose, str)
        self.assertIsInstance(DiscRecording.DRStatusStateSessionClose, str)
        self.assertIsInstance(DiscRecording.DRStatusStateFinishing, str)
        self.assertIsInstance(DiscRecording.DRStatusStateErasing, str)
        self.assertIsInstance(DiscRecording.DRStatusProgressInfoKey, str)
        self.assertIsInstance(DiscRecording.DRStatusProgressCurrentKPS, str)
        self.assertIsInstance(DiscRecording.DRStatusProgressCurrentXFactor, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusKey, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusErrorKey, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusErrorStringKey, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusErrorInfoStringKey, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusSenseKey, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusSenseCodeStringKey, str)
        self.assertIsInstance(DiscRecording.DRErrorStatusAdditionalSenseStringKey, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRCoreErrors(TestCase):
    def testConstants(self):
        self.assertEqual(DiscRecording.kDRFirstErr, 0x80020000)
        self.assertEqual(DiscRecording.kDRInternalErr, DiscRecording.kDRFirstErr)
        self.assertEqual(DiscRecording.kDRDeviceAccessErr, 0x80020020)
        self.assertEqual(DiscRecording.kDRDeviceBusyErr, 0x80020021)
        self.assertEqual(DiscRecording.kDRDeviceCommunicationErr, 0x80020022)
        self.assertEqual(DiscRecording.kDRDeviceInvalidErr, 0x80020023)
        self.assertEqual(DiscRecording.kDRDeviceNotReadyErr, 0x80020024)
        self.assertEqual(DiscRecording.kDRDeviceNotSupportedErr, 0x80020025)
        self.assertEqual(DiscRecording.kDRMediaBusyErr, 0x80020040)
        self.assertEqual(DiscRecording.kDRMediaNotPresentErr, 0x80020041)
        self.assertEqual(DiscRecording.kDRMediaNotWritableErr, 0x80020042)
        self.assertEqual(DiscRecording.kDRMediaNotSupportedErr, 0x80020043)
        self.assertEqual(DiscRecording.kDRMediaNotBlankErr, 0x80020044)
        self.assertEqual(DiscRecording.kDRMediaNotErasableErr, 0x80020045)
        self.assertEqual(DiscRecording.kDRMediaInvalidErr, 0x80020046)
        self.assertEqual(DiscRecording.kDRBurnUnderrunErr, 0x80020060)
        self.assertEqual(DiscRecording.kDRBurnNotAllowedErr, 0x80020061)
        self.assertEqual(DiscRecording.kDRDataProductionErr, 0x80020062)
        self.assertEqual(DiscRecording.kDRVerificationFailedErr, 0x80020063)
        self.assertEqual(DiscRecording.kDRTooManyTracksForDVDErr, 0x80020064)
        self.assertEqual(DiscRecording.kDRBadLayoutErr, 0x80020065)
        self.assertEqual(DiscRecording.kDRUserCanceledErr, 0x80020066)
        self.assertEqual(DiscRecording.kDRFunctionNotSupportedErr, 0x80020067)
        self.assertEqual(DiscRecording.kDRSpeedTestAlreadyRunningErr, 0x80020068)
        self.assertEqual(DiscRecording.kDRInvalidIndexPointsErr, 0x80020069)
        self.assertEqual(
            DiscRecording.kDRDoubleLayerL0DataZoneBlocksParamErr, 0x8002006A
        )
        self.assertEqual(DiscRecording.kDRDoubleLayerL0AlreadySpecifiedErr, 0x8002006B)
        self.assertEqual(DiscRecording.kDRAudioFileNotSupportedErr, 0x8002006C)
        self.assertEqual(DiscRecording.kDRBurnPowerCalibrationErr, 0x8002006D)
        self.assertEqual(DiscRecording.kDRBurnMediaWriteFailureErr, 0x8002006E)
        self.assertEqual(DiscRecording.kDRTrackReusedErr, 0x8002006F)
        self.assertEqual(DiscRecording.kDRFileModifiedDuringBurnErr, 0x80020100)
        self.assertEqual(DiscRecording.kDRFileLocationConflictErr, 0x80020101)
        self.assertEqual(DiscRecording.kDRTooManyNameConflictsErr, 0x80020102)
        self.assertEqual(DiscRecording.kDRFilesystemsNotSupportedErr, 0x80020103)
        self.assertEqual(DiscRecording.kDRDeviceBurnStrategyNotAvailableErr, 0x80020200)
        self.assertEqual(DiscRecording.kDRDeviceCantWriteCDTextErr, 0x80020201)
        self.assertEqual(DiscRecording.kDRDeviceCantWriteIndexPointsErr, 0x80020202)
        self.assertEqual(DiscRecording.kDRDeviceCantWriteISRCErr, 0x80020203)
        self.assertEqual(DiscRecording.kDRDeviceCantWriteSCMSErr, 0x80020204)
        self.assertEqual(DiscRecording.kDRDevicePreGapLengthNotValidErr, 0x80020205)

        self.assertIsInstance(DiscRecording.kDRErrorStatusKey, str)
        self.assertIsInstance(DiscRecording.kDRErrorStatusErrorKey, str)
        self.assertIsInstance(DiscRecording.kDRErrorStatusErrorStringKey, str)
        self.assertIsInstance(DiscRecording.kDRErrorStatusErrorInfoStringKey, str)
        self.assertIsInstance(DiscRecording.kDRErrorStatusSenseKey, str)
        self.assertIsInstance(DiscRecording.kDRErrorStatusSenseCodeStringKey, str)
        self.assertIsInstance(DiscRecording.kDRErrorStatusAdditionalSenseStringKey, str)

    def testFunctions(self):
        self.assertResultIsCFRetained(
            DiscRecording.DRCopyLocalizedStringForDiscRecordingError
        )
        self.assertResultIsCFRetained(DiscRecording.DRCopyLocalizedStringForSenseCode)
        self.assertResultIsCFRetained(
            DiscRecording.DRCopyLocalizedStringForAdditionalSense
        )



import DiscRecording
from PyObjCTools.TestSupport import TestCase
import objc


class TestDRTrackHelper(DiscRecording.NSObject):
    def estimateLengthOfTrack_(self, a):
        return 1

    def prepareTrack_forBurn_toMedia_(self, a, b, c):
        return 1

    def producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_(
        self, a, b, c, d, e, f
    ):
        return 1

    def produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_(
        self, a, b, c, d, e, f
    ):
        return 1

    def prepareTrackForVerification_(self, a):
        return 1

    def verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_(
        self, a, b, c, d, e, f
    ):
        return 1

    def verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_(
        self, a, b, c, d, e, f
    ):
        return 1

    def cleanupTrackAfterVerification_(self, a):
        return 1


class TestDRTrack(TestCase):
    def testProtocols(self):
        objc.protocolNamed("DRTrackDataProduction")

    def testMethods(self):
        self.assertResultHasType(
            TestDRTrackHelper.estimateLengthOfTrack_, objc._C_ULNG_LNG
        )
        self.assertResultIsBOOL(TestDRTrackHelper.prepareTrack_forBurn_toMedia_)

        self.assertResultHasType(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            b"o^v",
        )
        self.assertArgSizeInArg(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            2,
        )
        self.assertArgHasType(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            2,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            3,
            objc._C_ULNG_LNG,
        )
        self.assertArgHasType(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            4,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.producePreGapForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            5,
            b"N^I",
        )

        self.assertResultHasType(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            b"o^v",
        )
        self.assertArgSizeInArg(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            2,
        )
        self.assertArgHasType(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            2,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            3,
            objc._C_ULNG_LNG,
        )
        self.assertArgHasType(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            4,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.produceDataForTrack_intoBuffer_length_atAddress_blockSize_ioFlags_,
            5,
            b"N^I",
        )

        self.assertResultIsBOOL(TestDRTrackHelper.prepareTrackForVerification_)

        self.assertResultIsBOOL(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            b"n^v",
        )
        self.assertArgSizeInArg(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            2,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            2,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            3,
            objc._C_ULNG_LNG,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            4,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyPreGapForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            5,
            b"N^I",
        )

        self.assertResultIsBOOL(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            b"n^v",
        )
        self.assertArgSizeInArg(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            1,
            2,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            2,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            3,
            objc._C_ULNG_LNG,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            4,
            objc._C_UINT,
        )
        self.assertArgHasType(
            TestDRTrackHelper.verifyDataForTrack_inBuffer_length_atAddress_blockSize_ioFlags_,
            5,
            b"N^I",
        )

        self.assertResultIsBOOL(TestDRTrackHelper.cleanupTrackAfterVerification_)

    def testConstants(self):
        self.assertIsInstance(DiscRecording.DRTrackLengthKey, str)
        self.assertIsInstance(DiscRecording.DRBlockSizeKey, str)
        self.assertIsInstance(DiscRecording.DRBlockTypeKey, str)
        self.assertIsInstance(DiscRecording.DRDataFormKey, str)
        self.assertIsInstance(DiscRecording.DRSessionFormatKey, str)
        self.assertIsInstance(DiscRecording.DRTrackModeKey, str)
        self.assertIsInstance(DiscRecording.DRVerificationTypeKey, str)
        self.assertIsInstance(DiscRecording.DRMaxBurnSpeedKey, str)
        self.assertIsInstance(DiscRecording.DRPreGapLengthKey, str)
        self.assertIsInstance(DiscRecording.DRPreGapIsRequiredKey, str)
        self.assertIsInstance(DiscRecording.DRDVDTimestampKey, str)
        self.assertIsInstance(DiscRecording.DRDVDCopyrightInfoKey, str)
        self.assertIsInstance(DiscRecording.DRTrackISRCKey, str)
        self.assertIsInstance(DiscRecording.DRIndexPointsKey, str)
        self.assertIsInstance(DiscRecording.DRAudioPreEmphasisKey, str)
        self.assertIsInstance(DiscRecording.DRAudioFourChannelKey, str)
        self.assertIsInstance(DiscRecording.DRSerialCopyManagementStateKey, str)
        self.assertIsInstance(DiscRecording.DRVerificationTypeProduceAgain, str)
        self.assertIsInstance(DiscRecording.DRVerificationTypeReceiveData, str)
        self.assertIsInstance(DiscRecording.DRVerificationTypeChecksum, str)
        self.assertIsInstance(DiscRecording.DRVerificationTypeNone, str)
        self.assertIsInstance(DiscRecording.DRSCMSCopyrightFree, str)
        self.assertIsInstance(DiscRecording.DRSCMSCopyrightProtectedOriginal, str)
        self.assertIsInstance(DiscRecording.DRSCMSCopyrightProtectedCopy, str)
        self.assertIsInstance(DiscRecording.DRNextWritableAddressKey, str)
        self.assertIsInstance(DiscRecording.DRTrackStartAddressKey, str)
        self.assertIsInstance(DiscRecording.DRFreeBlocksKey, str)
        self.assertIsInstance(DiscRecording.DRTrackNumberKey, str)
        self.assertIsInstance(DiscRecording.DRSessionNumberKey, str)
        self.assertIsInstance(DiscRecording.DRTrackTypeKey, str)
        self.assertIsInstance(DiscRecording.DRTrackIsEmptyKey, str)
        self.assertIsInstance(DiscRecording.DRTrackPacketTypeKey, str)
        self.assertIsInstance(DiscRecording.DRTrackPacketSizeKey, str)
        self.assertIsInstance(DiscRecording.DRTrackTypeInvisible, str)
        self.assertIsInstance(DiscRecording.DRTrackTypeIncomplete, str)
        self.assertIsInstance(DiscRecording.DRTrackTypeReserved, str)
        self.assertIsInstance(DiscRecording.DRTrackTypeClosed, str)
        self.assertIsInstance(DiscRecording.DRTrackPacketTypeFixed, str)
        self.assertIsInstance(DiscRecording.DRTrackPacketTypeVariable, str)
        self.assertIsInstance(DiscRecording.DRISOLevel, str)
        self.assertIsInstance(DiscRecording.DRVolumeSet, str)
        self.assertIsInstance(DiscRecording.DRPublisher, str)
        self.assertIsInstance(DiscRecording.DRDataPreparer, str)
        self.assertIsInstance(DiscRecording.DRApplicationIdentifier, str)
        self.assertIsInstance(DiscRecording.DRSystemIdentifier, str)
        self.assertIsInstance(DiscRecording.DRCopyrightFile, str)
        self.assertIsInstance(DiscRecording.DRAbstractFile, str)
        self.assertIsInstance(DiscRecording.DRBibliographicFile, str)
        self.assertIsInstance(DiscRecording.DRBlockSize, str)
        self.assertIsInstance(DiscRecording.DRDefaultDate, str)
        self.assertIsInstance(DiscRecording.DRVolumeCreationDate, str)
        self.assertIsInstance(DiscRecording.DRVolumeModificationDate, str)
        self.assertIsInstance(DiscRecording.DRVolumeCheckedDate, str)
        self.assertIsInstance(DiscRecording.DRVolumeExpirationDate, str)
        self.assertIsInstance(DiscRecording.DRVolumeEffectiveDate, str)
        self.assertIsInstance(DiscRecording.DRISOMacExtensions, str)
        self.assertIsInstance(DiscRecording.DRISORockRidgeExtensions, str)
        self.assertIsInstance(DiscRecording.DRSuppressMacSpecificFiles, str)

        self.assertEqual(DiscRecording.DRFlagSubchannelDataRequested, 1 << 1)

        self.assertIsInstance(DiscRecording.DRSubchannelDataFormKey, str)
        self.assertIsInstance(DiscRecording.DRSubchannelDataFormNone, str)
        self.assertIsInstance(DiscRecording.DRSubchannelDataFormPack, str)
        self.assertIsInstance(DiscRecording.DRSubchannelDataFormRaw, str)



import DiscRecording
from PyObjCTools.TestSupport import TestCase


class TestDRContentTrack(TestCase):
    def testFunctions(self):
        self.assertResultIsCFRetained(DiscRecording.DRFilesystemTrackCreate)

        DiscRecording.DRFilesystemTrackEstimateOverhead

        self.assertResultIsCFRetained(DiscRecording.DRAudioTrackCreate)
        self.assertArgIsIn(DiscRecording.DRAudioTrackCreate, 0)

        self.assertResultIsCFRetained(DiscRecording.DRAudioTrackCreateWithURL)
